@@ -1,5 +1,5 @@
 /*	canvas.h
-	Copyright (C) 2004-2011 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2020 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -20,14 +20,14 @@
 float can_zoom;						// Zoom factor 1..MAX_ZOOM
 int margin_main_xy[2];					// Top left of image from top left of canvas
 int margin_view_xy[2];
-int zoom_flag;
 int marq_status, marq_xy[4];				// Selection marquee
 int marq_drag_x, marq_drag_y;				// Marquee dragging offset
 int line_status, line_xy[4];				// Line tool
 int poly_status;					// Polygon selection tool
-int clone_x, clone_y;					// Clone offsets
-int recent_files;					// Current recent files setting
+int clone_status, clone_x, clone_y, clone_dx, clone_dy;	// Clone tool state
+int clone_mode, clone_x0, clone_y0, clone_dx0, clone_dy0;	// Clone settings
 int brush_spacing;					// Step in non-continuous mode
+int lasso_sel;						// Lasso by selection channel
 
 #define margin_main_x margin_main_xy[0]
 #define margin_main_y margin_main_xy[1]
@@ -45,6 +45,10 @@ int brush_spacing;					// Step in non-continuous mode
 #define line_x2 line_xy[2]
 #define line_y2 line_xy[3]
 
+#define MAX_RECENT 20
+int recent_files;					// Current recent files setting
+char *recent_filenames[MAX_RECENT];			// Recent filenames themselves
+
 int preserved_gif_delay, undo_load;
 
 #define STATUS_ITEMS 5
@@ -54,14 +58,15 @@ int preserved_gif_delay, undo_load;
 #define STATUS_SELEGEOM 3
 #define STATUS_UNDOREDO 4
 
-GtkWidget *label_bar[STATUS_ITEMS];
+void **label_bar[STATUS_ITEMS];
 
 int	col_reverse,					// Painting with right button
 	show_paste,					// Show contents of clipboard while pasting
 	status_on[STATUS_ITEMS],			// Show status bar items?
 	text_paste,					// Are we pasting text?
 	canvas_image_centre,				// Are we centering the image?
-	chequers_optimize				// Are we optimizing the chequers for speed?
+	chequers_optimize,				// Are we optimizing the chequers for speed?
+	cursor_zoom					// Are we zooming at cursor position?
 	;
 
 #define LINE_NONE 0
@@ -84,8 +89,13 @@ int	col_reverse,					// Painting with right button
 #define GRAD_END 2
 #define GRAD_DONE 3
 
+#define CLONE_REL   0
+#define CLONE_ABS   1
+#define CLONE_TRACK 2 /* Reposition flag */
+#define CLONE_DRAG  4 /* Stroke flag */
+
 #define MIN_ZOOM 0.1
-#define MAX_ZOOM 20
+#define MAX_ZOOM 80
 
 /* File selector codes */
 enum {
@@ -111,22 +121,65 @@ enum {
 	FS_PALETTE_DEF
 };
 
-#define FS_ENTRY_KEY "mtPaint.fs_entry"
-
-int do_a_load(char *fname, int undo);
+int do_a_load_x(char *fname, int undo, void *v);
+#define do_a_load(A,B) do_a_load_x(A, B, NULL)
 void canvas_center(float ic[2]);
 void align_size(float new_zoom);
 void realign_size();
-void init_ls_settings(ls_settings *settings, GtkWidget *box);
+int ftype_selector(int mask, char *ext, int def, char **names, int *ftypes);
+int tiff_type_selector(int mask, int def, char **names);
+void init_ls_settings(ls_settings *settings, void **wdata);
 void fs_setup(GtkWidget *fs, int action_type);
-void file_selector( int action_type );
+void file_selector_x(int action_type, void **xdata);
+void file_selector(int action_type);
 void set_new_filename(int layer, char *fname);
 
-void main_undo();
-void main_redo();
+void pressed_do_undo(int redo);
 
 void line_to_gradient();	// Setup gradient along line tool
-void tool_action(int event, int x, int y, int button, gdouble pressure);	// Paint some pixels!
+
+/* Tool action codes */
+enum {
+	TC_NONE = 0,
+	TC_LINE_START,
+	TC_LINE_ARROW,
+	TC_LINE_NEXT,
+	TC_LINE_STOP,
+	TC_SEL_CLEAR,
+	TC_SEL_START,
+	TC_SEL_SET_0,
+	TC_SEL_SET_1,
+	TC_SEL_SET_2,
+	TC_SEL_SET_3,
+	TC_SEL_TO,
+	TC_SEL_STOP,
+	TC_POLY_START,
+	TC_POLY_START_D,
+	TC_POLY_ADD,
+	TC_POLY_DEL,
+	TC_POLY_CLOSE,
+	TC_PASTE_DRAG,
+	TC_PASTE_COMMIT,
+	TC_PASTE_PAINT,
+	TC_PASTE_PSWAP,
+	TC_PAINT,
+	TC_PAINT_B,
+	TC_GRAD_START,
+	TC_GRAD_SET0,
+	TC_GRAD_SET1,
+	TC_GRAD_PICK0,
+	TC_GRAD_PICK1,
+	TC_GRAD_DRAG0,
+	TC_GRAD_DRAG1,
+	TC_GRAD_CLEAR,
+};
+
+#define TC_OPMASK 0x0FF
+#define TCF_PRES  0x100 /* Use pressure */
+#define TCF_ONCE  0x200 /* Skip repeats */
+
+void do_tool_action(int cmd, int x, int y, int pressure);	// Paint some pixels!
+int tool_action(int count, int button, int x, int y);	// Decide what to paint
 void update_menus();					// Update undo/edit menu
 
 int close_to( int x1, int y1 );
@@ -138,10 +191,10 @@ int close_to( int x1, int y1 );
 #define MARQ_SIZE 4 /* +snap */
 
 void paint_marquee(int action, int new_x, int new_y, rgbcontext *ctx);	// Draw/clear marquee
-void paint_poly_marquee(rgbcontext *ctx, int whole);	// Paint polygon marquee
-void stretch_poly_line(int x, int y);			// Clear old temp line, draw next temp line
+void paint_poly_marquee(rgbcontext *ctx);	// Paint polygon marquee
+void stretch_poly_line(int x, int y);		// Clear old temp line, draw next temp line
 
-void update_sel_bar();			// Update selection stats on status bar
+void update_sel_bar(int now);		// Update selection stats on status bar
 void update_xy_bar(int x, int y);	// Update cursor tracking on status bar
 void init_status_bar();			// Initialize status bar
 
@@ -150,7 +203,8 @@ void pressed_copy(int cut);
 void pressed_paste(int centre);
 void pressed_greyscale(int mode);
 void pressed_convert_rgb();
-void pressed_invert();
+void pressed_inv(int what);
+void pressed_map();
 void pressed_rectangle(int filled);
 void pressed_ellipse(int filled);
 
@@ -188,9 +242,7 @@ void repaint_grad(const int *old);		// Repaint gradient line and maybe clear old
 void repaint_line(const int *old);		// Repaint line on canvas and maybe clear old one
 void refresh_line(int mode, const int *lxy, rgbcontext *ctx);	// Refresh a part of line/gradient if needed
 void register_file( char *filename );		// Called after successful load/save
-void update_recent_files();			// Update the menu items
-
-void update_all_views();			// Update whole canvas on all views
+void update_recent_files(int save);		// Update the menu items / save to inifile
 
 void create_default_image();			// Create default new image
 
@@ -221,15 +273,16 @@ void create_default_image();			// Create default new image
 #define CF_ALIGN  0x00100000 /* Realign image window */
 #define CF_VALIGN 0x00200000 /* Realign view window */
 #define CF_TRANS  0x00400000 /* Transparent color in layers window */
+#define CF_NOW    0x00800000 /* Do not wait for cumulative update */
 
 /* Compound updates */
 
 //	Changed image contents
 #define UPD_IMG    (CF_DRAW | CF_VDRAW | CF_PIXEL)
 //	Changed image geometry (+undo)
-#define UPD_GEOM   (CF_GEOM | CF_MENU | CF_IMGBAR | UPD_IMG)
+#define UPD_GEOM   (CF_GEOM | CF_MENU | CF_IMGBAR | UPD_IMG | UPD_SGEOM)
 //	Added a new channel (+)
-#define UPD_ADDCH  (CF_MENU | CF_IMGBAR | UPD_IMG)
+#define UPD_ADDCH  (CF_SET | CF_MENU | CF_IMGBAR | UPD_IMG)
 //	Deleted an existing channel (+)
 #define UPD_DELCH  UPD_ADDCH
 //	Switched to new channel (+)
@@ -239,7 +292,7 @@ void create_default_image();			// Create default new image
 //	Changed color or value A or B
 #define UPD_CAB    CF_CAB
 //	Changed color A or B
-#define UPD_AB     (CF_AB | CF_GRAD | CF_SET | CF_GMODE | CF_TDRAW)
+#define UPD_AB     (CF_AB | CF_GRAD | CF_SET | CF_GMODE | CF_TDRAW | CF_PDRAW)
 //	Changed pattern
 #define UPD_PAT    (CF_AB | CF_TDRAW)
 //	Changed A, B, and pattern
@@ -248,8 +301,6 @@ void create_default_image();			// Create default new image
 #define UPD_PAL    (CF_PAL | CF_IMGBAR | CF_PDRAW | UPD_AB | UPD_IMG)
 //	Changed palette and transparent color
 #define UPD_TPAL   (UPD_PAL | CF_TRANS)
-//	Added colors to palette
-#define UPD_ADDPAL (CF_PAL | CF_IMGBAR | CF_PDRAW)
 //	Changed drawing mode in some way
 #define UPD_MODE   (CF_PMODE | CF_GMODE)
 //	Toggled gradient mode
@@ -272,6 +323,10 @@ void create_default_image();			// Create default new image
 #define UPD_CLIP   CF_PMODE
 //	Changed rendering options
 #define UPD_RENDER CF_DRAW
+//	Changed viewing options
+#define UPD_VIEW   CF_VDRAW
+//	Update all views
+#define UPD_ALLV   (CF_DRAW | CF_VDRAW)
 //	Changed clipboard geometry
 #define UPD_CGEOM  (CF_CGEOM | CF_SELBAR)
 //	Changed polygonal selection (-redraw)
