@@ -1,5 +1,5 @@
 /*	help.c
-	Copyright (C) 2004-2011 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2016 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -29,7 +29,7 @@ _("Credits"),
 };
 
 static char *help_page0[] = {
-_("mtPaint 3.40 - Copyright (C) 2004-2011 The Authors\n"),
+_("mtPaint 3.50 - Copyright (C) 2004-2016 The Authors\n"),
 _("See 'Credits' section for a list of the authors.\n"),
 _("mtPaint is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.\n"),
 _("mtPaint is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n"),
@@ -83,6 +83,7 @@ _("  Shift-T           Text Tool (GTK+)"),
 _("  T                 Text Tool (FreeType)\n"),
 _("  V                 View Window"),
 _("  L                 Layers Window\n"),
+_("  Ctrl-H            Toggle Hide Image mode"),
 _("  B                 Toggle Snap to Tile Grid mode\n"),
 _("  X                 Swap Colours A & B"),
 _("  E                 Choose Colour\n"),
@@ -133,8 +134,10 @@ _("  Middle button        Selects the point which will be the centre of the imag
 _("  Right button         Commit paste to canvas / Stop drawing current line / Cancel selection\n"),
 _("  Scroll Wheel         In GTK+2 the user can have the scroll wheel zoom in or out via the Preferences window\n"),
 _("  Ctrl+Left button     Choose colour A from under mouse pointer"),
+_("  Ctrl+Left button     Clone tool - Select source"),
 _("  Ctrl+Middle button   Create colour A/B and pattern based on the RGB colour in A (RGB images only)"),
 _("  Ctrl+Right button    Choose colour B from under mouse pointer"),
+_("  Ctrl+Right button    Polygon tool - Delete last point / Delete nearest point"),
 _("  Ctrl+Scroll Wheel    Scroll the main edit window left or right\n"),
 _("  Ctrl+Double click    Set colour A or B to average colour under brush square or selection marquee (RGB only)\n"),
 _("  Shift+Right button   Selects the point which will be the centre of the image after the next zoom\n\n"),
@@ -156,6 +159,7 @@ _("Abdulla Al Muhairi - Website redesign April 2005"),
 _("Alan Horkan"),
 _("Alexandre Prokoudine"),
 _("Antonio Andrea Bianco"),
+_("Charlie Ledocq"),
 _("Dennis Lee"),
 _("Donald White"),
 _("Ed Jason"),
@@ -167,6 +171,7 @@ _("Martin Zelaia"),
 _("Pasi Kallinen"),
 _("Pavel Ruzicka"),
 _("Puppy Linux (Barry Kauler)"),
+_("Victor Copovi"),
 _("Vlastimil Krejcir"),
 _("William Kern\n\n"),
 _("Translations\n"),
@@ -190,10 +195,11 @@ _("Tagalog - Anjelo delCarmen"),
 _("Taiwanese Chinese - Wei-Lun Chao"),
 _("Turkish - Muhammet Kara, Tutku Dalmaz"),
 NULL };
-#define HELP_PAGE_MAX 88
+
+#define HELP_PAGE_MAX 89
 
 static char **help_pages[HELP_PAGE_COUNT] = {
-	help_page0, help_page1, help_page2, help_page3
+	help_page0, help_page1, help_page2, help_page3, 
 };
 
 #undef _
