@@ -1,5 +1,5 @@
 /*	mygtk.h
-	Copyright (C) 2004-2011 Mark Tyler and Dmitry Groshev
+	Copyright (C) 2004-2020 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -37,13 +37,58 @@
 #endif
 #endif
 
+///	List widgets to use
+
+#if GTK_MAJOR_VERSION == 1
+#undef U_LISTS_GTK1
+#define U_LISTS_GTK1
+
+#elif GTK_MAJOR_VERSION == 2
+#if GTK2VERSION < 18
+#undef U_LISTS_GTK1
+#define U_LISTS_GTK1
+#endif
+
+#else /* GTK_MAJOR_VERSION == 3 */
+#undef U_LISTS_GTK1
+#endif
+
+///	Meaningless differences to hide
+
+#if GTK_MAJOR_VERSION == 3
+#define g_thread_init(A) /* Not needed anymore */
+#define GtkSignalFunc GCallback
+#define GTK_SIGNAL_FUNC(A) G_CALLBACK(A)
+#define GTK_OBJECT(A) G_OBJECT(A)
+#define gtk_signal_connect(A,B,C,D) g_signal_connect(A,B,C,D)
+#define gtk_signal_connect_object(A,B,C,D) g_signal_connect_swapped(A,B,C,D)
+#define gtk_signal_disconnect_by_data(A,B) g_signal_handlers_disconnect_by_data(A,B)
+#define gtk_signal_emit_by_name g_signal_emit_by_name
+
+#define KEY(A) GDK_KEY_##A
+
+#else
+#define gtk_widget_get_parent(A) ((A)->parent)
+#define gtk_widget_get_window(A) ((A)->window)
+#define gdk_device_get_name(A) ((A)->name)
+#define gdk_device_get_n_axes(A) ((A)->num_axes)
+#define gdk_device_get_mode(A) ((A)->mode)
+#define KEY(A) GDK_##A
+#endif
+
+#ifndef G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+#define G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+#define G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
+
 ///	Icon descriptor type
 
 typedef void *xpm_icon_desc[2];
 
 #if GTK_MAJOR_VERSION == 1
 #define XPM_TYPE char**
-#else /* if GTK_MAJOR_VERSION == 2 */
+
+#else /* if GTK_MAJOR_VERSION >= 2 */
 #define XPM_TYPE void**
 #endif
 
@@ -54,24 +99,16 @@ typedef struct {
 	unsigned char *rgb;
 } rgbcontext;
 
+///	Main toplevel, for anchoring dialogs and rendering pixmaps
+
+GtkWidget *main_window;
+
 ///	Generic Widget Primitives
 
-GtkWidget *add_a_window( GtkWindowType type, char *title, GtkWindowPosition pos, gboolean modal );
-GtkWidget *add_a_button( char *text, int bord, GtkWidget *box, gboolean filler );
+GtkWidget *add_a_window(GtkWindowType type, char *title, GtkWindowPosition pos);
 GtkWidget *add_a_spin( int value, int min, int max );
-GtkWidget *add_a_table( int rows, int columns, int bord, GtkWidget *box );
-GtkWidget *add_a_toggle( char *label, GtkWidget *box, gboolean value );
-GtkWidget *add_to_table( char *text, GtkWidget *table, int row, int column, int spacing);
-GtkWidget *add_to_table_l(char *text, GtkWidget *table, int row, int column,
-	int l, int spacing);
-GtkWidget *to_table(GtkWidget *widget, GtkWidget *table, int row, int column, int spacing);
-GtkWidget *to_table_l(GtkWidget *widget, GtkWidget *table, int row, int column,
-	int l, int spacing);
-GtkWidget *spin_to_table( GtkWidget *table, int row, int column, int spacing,
-	int value, int min, int max );
-GtkWidget *float_spin_to_table(GtkWidget *table, int row, int column, int spacing,
-	double value, double min, double max);
-void add_hseparator( GtkWidget *widget, int xs, int ys );
+
+int user_break;
 
 void progress_init(char *text, int canc);		// Initialise progress window
 int progress_update(float val);				// Update progress window
@@ -79,39 +116,46 @@ void progress_end();					// Close progress window
 
 int alert_box(char *title, char *message, char *text1, ...);
 
-// Add page to notebook
+//	Tablet handling
 
-GtkWidget *add_new_page(GtkWidget *notebook, char *name);
+#if GTK_MAJOR_VERSION == 1
+GdkDeviceInfo *tablet_device;
+#else /* #if GTK_MAJOR_VERSION >= 2 */
+GdkDevice *tablet_device;
+#endif
+void init_tablet();
+void conf_tablet(void **slot);
+void conf_done(void *cause);
 
-// Slider-spin combo (practically a new widget class)
+// GTK+3 specific support code
+
+#if GTK_MAJOR_VERSION == 3
+void cairo_surface_fdestroy(cairo_surface_t *s);
+cairo_surface_t *cairo_upload_rgb(cairo_surface_t *ref, GdkWindow *win,
+	unsigned char *src, int w, int h, int len);
+void cairo_set_rgb(cairo_t *cr, int c);
+void css_restyle(GtkWidget *w, char *css, char *class, char *name);
+void add_css_class(GtkWidget *w, char *class);
+/* Add CSS, builtin and user-provided, to default screen */
+void init_css(char *cssfile);
+/* Find button widget of a GtkComboBox with entry */
+GtkWidget *combobox_button(GtkWidget *cbox);
+/* Find out which set of bugs & breakages is active */
+int gtk3version;
+#else
+#define add_css_class(A,B)
+#endif
+
+// Slider-spin combo (a decorated spinbutton)
 
 GtkWidget *mt_spinslide_new(int swidth, int sheight);
-void mt_spinslide_set_range(GtkWidget *spinslide, int minv, int maxv);
-int mt_spinslide_get_value(GtkWidget *spinslide);
-int mt_spinslide_read_value(GtkWidget *spinslide);
-void mt_spinslide_set_value(GtkWidget *spinslide, int value);
-/* void handler(GtkAdjustment *adjustment, gpointer user_data); */
-void mt_spinslide_connect(GtkWidget *spinslide, GtkSignalFunc handler,
-	gpointer user_data);
-#define SPINSLIDE_ADJUSTMENT(s) \
-	(GTK_SPIN_BUTTON(BOX_CHILD_1(s))->adjustment)
 #define ADJ2INT(a) ((int)rint((a)->value))
 
 // Self-contained package of radio buttons
 
-GtkWidget *wj_radio_pack(char **names, int cnt, int vnum, int idx, gpointer var,
+GtkWidget *wj_radio_pack(char **names, int cnt, int vnum, int idx, void **r,
 	GtkSignalFunc handler);
-
-// Convert window close into a button click ("Cancel" or whatever)
-
-void delete_to_click(GtkWidget *window, GtkWidget *button);
-
-// Buttons for standard dialogs
-
-GtkWidget *OK_box(int border, GtkWidget *window, char *nOK, GtkSignalFunc OK,
-	char *nCancel, GtkSignalFunc Cancel);
-GtkWidget *OK_box_add(GtkWidget *box, char *name, GtkSignalFunc Handler);
-GtkWidget *OK_box_add_toggle(GtkWidget *box, char *name, GtkSignalFunc Handler);
+int wj_radio_pack_get_active(GtkWidget *widget);
 
 // Easier way with spinbuttons
 
@@ -127,6 +171,7 @@ void spin_set_range(GtkWidget *spin, int min, int max);
 #endif
 
 // Box unpacking macros
+#if GTK_MAJOR_VERSION <= 2
 #define BOX_CHILD_0(box) \
 	(((GtkBoxChild*)GTK_BOX(box)->children->data)->widget)
 #define BOX_CHILD_1(box) \
@@ -135,6 +180,7 @@ void spin_set_range(GtkWidget *spin, int min, int max);
 	(((GtkBoxChild*)GTK_BOX(box)->children->next->next->data)->widget)
 #define BOX_CHILD(box, n) \
 	(((GtkBoxChild *)g_list_nth_data(GTK_BOX(box)->children, (n)))->widget)
+#endif
 
 // Wrapper for utf8->C and C->utf8 translation
 
@@ -159,36 +205,6 @@ char *wjstrcat(char *dest, int max, const char *s0, int l, ...);
 char *file_in_dir(char *dest, const char *dir, const char *file, int cnt);
 char *file_in_homedir(char *dest, const char *file, int cnt);
 
-// Extracting widget from GtkTable
-
-GtkWidget *table_slot(GtkWidget *table, int row, int col);
-
-// Packing framed widget
-
-GtkWidget *add_with_frame_x(GtkWidget *box, char *text, GtkWidget *widget,
-	int border, int expand);
-GtkWidget *add_with_frame(GtkWidget *box, char *text, GtkWidget *widget);
-
-// Entry + Browse
-
-GtkWidget *mt_path_box(char *name, GtkWidget *box, char *title, int fsmode);
-
-// Option menu
-
-GtkWidget *wj_option_menu(char **names, int cnt, int idx, gpointer var,
-	GtkSignalFunc handler);
-int wj_option_menu_get_history(GtkWidget *optmenu);
-
-// Workaround for broken option menu sizing in GTK2
-#if GTK_MAJOR_VERSION == 2
-void wj_option_realize(GtkWidget *widget, gpointer user_data);
-#define FIX_OPTION_MENU_SIZE(opt) \
-	gtk_signal_connect_after(GTK_OBJECT(opt), "realize", \
-		GTK_SIGNAL_FUNC(wj_option_realize), NULL)
-#else
-#define FIX_OPTION_MENU_SIZE(opt)
-#endif
-
 // Set minimum size for a widget
 
 void widget_set_minsize(GtkWidget *widget, int width, int height);
@@ -198,52 +214,20 @@ GtkWidget *widget_align_minsize(GtkWidget *widget, int width, int height);
 
 void widget_set_keepsize(GtkWidget *widget, int keep_height);
 
-// Signalled toggles
-
-GtkWidget *sig_toggle(char *label, int value, gpointer var, GtkSignalFunc handler);
-GtkWidget *sig_toggle_button(char *label, int value, gpointer var, GtkSignalFunc handler);
-
 // Workaround for GtkCList reordering bug in GTK2
 
 void clist_enable_drag(GtkWidget *clist);
 
-// Move browse-mode selection in GtkCList without invoking callbacks
-
-void clist_reselect_row(GtkCList *clist, int n);
-
-// Move browse-mode selection in GtkList
-
-void list_select_item(GtkWidget *list, GtkWidget *item);
-
-// Properly destroy transient window
-
-void destroy_dialog(GtkWidget *window);
-
-// Settings notebook
-
-GtkWidget *plain_book(GtkWidget **pages, int npages);
-GtkWidget *buttoned_book(GtkWidget **page0, GtkWidget **page1,
-	GtkWidget **button, char *button_label);
-
 // Most common use of boxes
 
 GtkWidget *pack(GtkWidget *box, GtkWidget *widget);
 GtkWidget *xpack(GtkWidget *box, GtkWidget *widget);
 GtkWidget *pack_end(GtkWidget *box, GtkWidget *widget);
-GtkWidget *pack5(GtkWidget *box, GtkWidget *widget);
-GtkWidget *xpack5(GtkWidget *box, GtkWidget *widget);
-GtkWidget *pack_end5(GtkWidget *box, GtkWidget *widget);
 
 // Put vbox into container
 
 GtkWidget *add_vbox(GtkWidget *cont);
 
-// Save/restore window positions
-
-void win_store_pos(GtkWidget *window, char *inikey);
-void win_restore_pos(GtkWidget *window, char *inikey, int defx, int defy,
-	int defw, int defh);
-
 // Fix for paned widgets losing focus in GTK+1
 
 #if GTK_MAJOR_VERSION == 1
@@ -268,46 +252,83 @@ guint keyval_key(guint keyval);
 
 // Interpreting arrow keys
 
-int arrow_key(GdkEventKey *event, int *dx, int *dy, int mult);
+int arrow_key_(unsigned key, unsigned state, int *dx, int *dy, int mult);
+#define arrow_key(E,X,Y,M) arrow_key_((E)->keyval, (E)->state, (X), (Y), (M))
 
 // Create pixmap cursor
 
-GdkCursor *make_cursor(const char *icon, const char *mask, int w, int h,
-	int tip_x, int tip_y);
+GdkCursor *make_cursor(char *icon, char *mask, int w, int h, int tip_x, int tip_y);
 
 // Menu-like combo box
 
-GtkWidget *wj_combo_box(char **names, int cnt, int idx, gpointer var,
+GtkWidget *wj_combo_box(char **names, int cnt, int u, int idx, void **r,
 	GtkSignalFunc handler);
 int wj_combo_box_get_history(GtkWidget *combobox);
 
+#if GTK_MAJOR_VERSION == 3
+// Bin widget with customizable size handling
+
+GtkWidget *wjsizebin_new(GCallback get_size, GCallback size_alloc, gpointer user_data);
+#else
 // Box widget with customizable size handling
 
 GtkWidget *wj_size_box();
+#endif
 
 // Disable visual updates while tweaking container's contents
 
 gpointer toggle_updates(GtkWidget *widget, gpointer unlock);
 
+// Maximized state
+
+#if GTK_MAJOR_VERSION == 1
+int is_maximized(GtkWidget *window);
+void set_maximized(GtkWidget *window);
+#elif GTK_MAJOR_VERSION == 2
+#define is_maximized(W) \
+	(!!(gdk_window_get_state((W)->window) & GDK_WINDOW_STATE_MAXIMIZED))
+#define set_maximized(W) gtk_window_maximize(W)
+#else /* if GTK_MAJOR_VERSION == 3 */
+#define is_maximized(W) gtk_window_is_maximized(GTK_WINDOW(W))
+#define set_maximized(W) gtk_window_maximize(W)
+#endif
+
 // Drawable to RGB
 
+#if GTK_MAJOR_VERSION == 3
+unsigned char *wj_get_rgb_image(GdkWindow *window, cairo_surface_t *s,
+	unsigned char *buf, int x, int y, int width, int height);
+#else /* if GTK_MAJOR_VERSION <= 2 */
 unsigned char *wj_get_rgb_image(GdkWindow *window, GdkPixmap *pixmap,
 	unsigned char *buf, int x, int y, int width, int height);
+#endif
 
 // Clipboard
 
 int internal_clipboard(int which);
-int process_clipboard(int which, char *what, GtkSignalFunc handler, gpointer data);
-int offer_clipboard(int which, GtkTargetEntry *targets, int ntargets,
-	GtkSignalFunc handler);
 
-// Allocate a memory chunk which is freed along with a given widget
+// Clipboard pixmaps
 
-void *bound_malloc(GtkWidget *widget, int size);
+typedef unsigned long XID_type;
 
-// Gamma correction toggle
+typedef struct {
+	int w, h, depth;
+	XID_type xid;
+#if GTK_MAJOR_VERSION == 3
+	cairo_surface_t *pm;
+#else /* if GTK_MAJOR_VERSION <= 2 */
+	GdkPixmap *pm;
+#endif
+} pixmap_info;
 
-GtkWidget *gamma_toggle();
+#if (GTK_MAJOR_VERSION == 1) || defined GDK_WINDOWING_X11
+#define HAVE_PIXMAPS
+int export_pixmap(pixmap_info *p, int w, int h);
+void pixmap_put_rows(pixmap_info *p, unsigned char *src, int y, int cnt);
+#endif
+int import_pixmap(pixmap_info *p, XID_type *xid); // xid = NULL for a screenshot
+void drop_pixmap(pixmap_info *p);
+int pixmap_get_rows(pixmap_info *p, unsigned char *dest, int y, int cnt);
 
 // Image widget
 
@@ -318,7 +339,7 @@ GtkWidget *xpm_image(XPM_TYPE xpm);
 /* !!! Mask needs be zeroed before the call - especially with GTK+1 :-) */
 #if GTK_MAJOR_VERSION == 1
 #define render_stock_pixmap(X,Y,Z) NULL
-#else
+#elif GTK_MAJOR_VERSION == 2
 GdkPixmap *render_stock_pixmap(GtkWidget *widget, const gchar *stock_id,
 	GdkBitmap **mask);
 #endif
@@ -330,45 +351,46 @@ int release_grab();
 // Frame widget with passthrough scrolling
 
 GtkWidget *wjframe_new();
-void add_with_wjframe(GtkWidget *bin, GtkWidget *widget);
 
 // Scrollable canvas widget
 
 GtkWidget *wjcanvas_new();
+void wjcanvas_set_expose(GtkWidget *widget, GtkSignalFunc handler, gpointer user_data);
 void wjcanvas_size(GtkWidget *widget, int width, int height);
 void wjcanvas_get_vport(GtkWidget *widget, int *vport);
 int wjcanvas_scroll_in(GtkWidget *widget, int x, int y);
 int wjcanvas_bind_mouse(GtkWidget *widget, GdkEventMotion *event, int x, int y);
+#if GTK_MAJOR_VERSION == 3
+void wjcanvas_uncache(GtkWidget *widget, int *rxy);
+void wjcanvas_draw_rgb(GtkWidget *widget, int x, int y, int w, int h,
+	unsigned char *rgb, int step, int fill, int repaint);
+#else /* if GTK_MAJOR_VERSION <= 2 */
+#define wjcanvas_uncache(A,B)
+#endif
 
 // Focusable pixmap widget
 
 GtkWidget *wjpixmap_new(int width, int height);
+#if GTK_MAJOR_VERSION == 3
+cairo_surface_t *wjpixmap_pixmap(GtkWidget *widget);
+#else /* if GTK_MAJOR_VERSION <= 2 */
 GdkPixmap *wjpixmap_pixmap(GtkWidget *widget);
+#endif
 void wjpixmap_draw_rgb(GtkWidget *widget, int x, int y, int w, int h,
 	unsigned char *rgb, int step);
-void wjpixmap_fill_rgb(GtkWidget *widget, int x, int y, int w, int h, int rgb);
 void wjpixmap_move_cursor(GtkWidget *widget, int x, int y);
 void wjpixmap_set_cursor(GtkWidget *widget, char *image, char *mask,
 	int width, int height, int hot_x, int hot_y, int focused);
-void wjpixmap_cursor(GtkWidget *widget, int *x, int *y);
 int wjpixmap_rxy(GtkWidget *widget, int x, int y, int *xr, int *yr);
 
-// Repaint expose region
+// Type of pathname
 
-// !!! For now, repaint_func() is expected to know widget & window to repaint
-typedef void (*repaint_func)(int x, int y, int w, int h);
+#define PT_ABS 0	/* Absolute */
+#define PT_REL 1	/* Relative */
+#define PT_DRIVE_ABS 2	/* On Windows: absolute w/o drive (\DIR\FILE) */
+#define PT_DRIVE_REL 3	/* On Windows: relative with drive (C:FILE) */
 
-#if GTK_MAJOR_VERSION == 1 /* No regions there */
-#define repaint_expose(event, vport, repaint, cost) \
-	(repaint)((event)->area.x + (vport)[0], (event)->area.y + (vport)[1], \
-		(event)->area.width, (event)->area.height)
-#else 
-void repaint_expose(GdkEventExpose *event, int *vport, repaint_func repaint, int cost);
-#endif
-
-// Track updates of multiple widgets (by whatever means necessary)
-
-void track_updates(GtkSignalFunc handler, GtkWidget *widget, ...);
+int path_type(char *path);
 
 // Convert pathname to absolute
 
@@ -386,10 +408,6 @@ int wjfnmatch(const char *mask, const char *str, int utf);
 void reseparate(char *str);
 #endif
 
-// Prod the focused spinbutton, if any, to finally update its value
-
-void update_window_spin(GtkWidget *window);
-
 // Process event queue
 
 void handle_events();
@@ -421,13 +439,32 @@ void vport_noshadow_fix(GtkWidget *widget);
 #define vport_noshadow_fix(X)
 #endif
 
+// Helper for accessing scrollbars
+
+void get_scroll_adjustments(GtkWidget *win, GtkAdjustment **h, GtkAdjustment **v);
+
+// Helper for widget show/hide
+
+void widget_showhide(GtkWidget *widget, int what);
+
+// Color name to value
+
+int parse_color(char *what);
+
+//	DPI value
+
+double window_dpi(GtkWidget *win);
+
+//	Memory size (Mb)
+
+unsigned sys_mem_size();
 
 // Filtering bogus xine-ui "keypresses" (Linux only)
 #ifdef WIN32
 #define XINE_FAKERY(key) 0
 #else
-#define XINE_FAKERY(key) (((key) == GDK_Shift_L) || ((key) == GDK_Control_L) \
-	|| ((key) == GDK_Scroll_Lock) || ((key) == GDK_Num_Lock))
+#define XINE_FAKERY(key) (((key) == KEY(Shift_L)) || ((key) == KEY(Control_L)) \
+	|| ((key) == KEY(Scroll_Lock)) || ((key) == KEY(Num_Lock)))
 #endif
 
 // Workaround for stupid GTK1 typecasts
@@ -491,7 +528,14 @@ guint threads_timeout_add(guint32 interval, GSourceFunc function, gpointer data)
 #define THREADS_ENTER() gdk_threads_enter()
 #define THREADS_LEAVE() gdk_threads_leave()
 #else
+#if GTK_MAJOR_VERSION == 3
+#define GTK_PRIORITY_REDRAW GDK_PRIORITY_REDRAW
+#define GtkFunction GSourceFunc
+#define threads_idle_add_priority(X,Y,Z) g_idle_add_full(X,Y,Z,NULL)
+#define gtk_idle_remove(A) g_source_remove(A)
+#else
 #define threads_idle_add_priority(X,Y,Z) gtk_idle_add_priority(X,Y,Z)
+#endif
 #define threads_timeout_add(X,Y,Z) g_timeout_add(X,Y,Z)
 #define THREADS_ENTER()
 #define THREADS_LEAVE()
