@@ -1,5 +1,5 @@
 /*	prefs.h
-	Copyright (C) 2005-2009 Mark Tyler
+	Copyright (C) 2005-2020 Mark Tyler and Dmitry Groshev
 
 	This file is part of mtPaint.
 
@@ -22,9 +22,13 @@
 #define DEFAULT_PAL_INI "defaultPalette"
 #define DEFAULT_PAT_INI "defaultPatterns"
 #define DEFAULT_THEME_INI "defaultTheme"
+#define DEFAULT_CSS_INI "defaultCSS"
 
-int tablet_working, tablet_tool_use[3];		// Size, flow, opacity
-float tablet_tool_factor[3];			// Size, flow, opacity
+#define MAX_TF 100 /* Tablet tool factor scale */
+
+int tablet_tool_use[3];				// Size, flow, opacity
+int tablet_tool_factor[3];			// Size, flow, opacity
 
 void pressed_preferences();
-void init_tablet();				// Set up variables
+
+void keys_selector();
